library editablegrid;

import 'package:flutter/material.dart';
import 'models/grid-headers.dart';
import 'utils/editable-cell-widgets.dart';
import 'widgets/editable/editable-cell.dart';

///Grid of editable items.
class EditableGridWidget extends StatefulWidget {
  ///Headers of the grid.
  final EditableGridHeaders headers;

  ///List of grid rows.
  ///
  ///Each row is a list of editable cells.
  final List<List<EditableGridCells>> gridCell;

  ///Height of the cells.
  final double cellHeight;

  EditableGridWidget(
      {Key key, this.headers, this.gridCell, this.cellHeight = 80})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => EditableGridWidgetState();
}

class EditableGridWidgetState extends State<EditableGridWidget> {
  _initialize() {
    //Check specified headers
    if (widget.headers.specifiedHeaders == GridHeaders.Both &&
        (widget.headers.columnHeaders == null ||
            widget.headers.rowHeaders == null))
      throw Exception("Column and row headers need to be specified.");
    if (widget.headers.specifiedHeaders == GridHeaders.Columns &&
        widget.headers.columnHeaders == null)
      throw Exception("Column headers need to be specified.");
    if (widget.headers.specifiedHeaders == GridHeaders.Rows &&
        widget.headers.rowHeaders == null)
      throw Exception("Row headers need to be specified.");

    //Verify content lenghts
    //If column headers were specified then the amount of cells per row should match the column collection lenght
    if (widget.headers.columnHeaders != null &&
        widget.headers.columnHeaders.length > 0 &&
        widget.gridCell
            .any((c) => c.length != widget.headers.columnHeaders.length))
      throw Exception("Unexpected length for some of the rows");
    //If row headers were specified then the amount of grid cells groups should match the row collection lenght
    if (widget.headers.rowHeaders != null &&
        widget.headers.rowHeaders.length > 0 &&
        widget.gridCell.length != widget.headers.rowHeaders.length)
      throw Exception("Unexpected length for some of the rows");
  }

  @override
  Widget build(BuildContext context) {
    //Perform initial validations
    _initialize();

    List<Widget> _listViewChildren = List<Widget>();

    //If the column headers were specified then its corresponding row should be the first one on the grid
    if (widget.headers.specifiedHeaders != GridHeaders.Rows) {
      //If both header types were specified then there should be an empty cell on the top left corner of the grid
      List<Widget> _columns =
          widget.headers.specifiedHeaders == GridHeaders.Both
              ? [HeaderWidget(label: "")]
              : [];
      //Add column headers
      _columns += widget.headers.columnHeaders;
      _listViewChildren.add(
        Container(
          child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: _columns),
        ),
      );
    }
    //Fill grid cells
    var rowCount = (widget.headers.rowHeaders == null ||
            widget.headers.rowHeaders.length == 0)
        ? widget.gridCell.length
        : widget.headers.rowHeaders.length;
    for (var i = 0; i < rowCount; i++) {
      //If specified, the first element of each row should be the row header
      List<Widget> _item =
          widget.headers.specifiedHeaders == GridHeaders.Columns
              ? widget.gridCell[i]
              : [widget.headers.rowHeaders[i]] + widget.gridCell[i];
      //Add body cells
      _listViewChildren.add(
        Container(
          child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: _item),
        ),
      );
    }

    return Column(
      children: buildChildren(_listViewChildren),
    );
  }

  ///Build grid.
  ///
  ///Each grid row will be separated by a divider.
  List<Widget> buildChildren(List<Widget> listViewChildren) {
    List<Widget> _children = List<Widget>();

    for (var i = 0; i < listViewChildren.length; i++) {
      var _child = listViewChildren[i];
      //Add child to the list
      _children.add(Container(child: _child));
      //Separate each child using a divider, unless it is the last item
      if (i < listViewChildren.length - 1) {
        _children.add(Divider(height: 5));
      }
    }

    return _children;
  }
}
